#include <c_types.h>
#include <osapi.h>
#include <user_interface.h>

/* local defines... */
#define VE_TASKPRIO_LOW    0
#define VE_TASKPRIO_MED    1
#define VE_TASKPRIO_HIGH   2

#define VE_TIMER_NOREPEAT  0
#define VE_TIMER_REPEAT    1

#define __TIMER_INTERVAL   2000

/* local helpers... */
#define __asize(x)  (sizeof(x) / sizeof(x[0]))
#define __d(x) { os_printf("%s:%u %s\n", __FUNCTION__, __LINE__, x); }

/* missing declarations in SDK-includes */
extern void os_printf_plus(const char *format, ...);
extern void ets_timer_setfn(os_timer_t *, os_timer_func_t *, void *);
extern void ets_timer_arm_new(os_timer_t *, uint32_t, int, int);
extern void uart_div_modify(int uart_no, int baudrate);

/* local types... */
enum __local_event_t {
    LOCAL_EVENT_INIT,
    LOCAL_EVENT_EXEC
};


static void ICACHE_FLASH_ATTR
__local_event_exec() {

    /* and finally what we are after for */
    __d("EXEC");
}

static void ICACHE_FLASH_ATTR
__local_timer_callback(void *arg) {

    /* send the actual SAYHELLO -message */
    system_os_post(VE_TASKPRIO_LOW, LOCAL_EVENT_EXEC, 0);
}

static void ICACHE_FLASH_ATTR
__local_event_init() {

    static os_timer_t __timer;

    /* set timer for periodic invokations */
    os_timer_setfn(&__timer, __local_timer_callback,
		   NULL /* arg to callback */);
    os_timer_arm(&__timer, __TIMER_INTERVAL, VE_TIMER_REPEAT);
}

static void ICACHE_FLASH_ATTR
__local_event_callback(os_event_t *ev) {

    enum __local_event_t event =
	(enum __local_event_t)ev->sig;
    
    switch(event) {
    case LOCAL_EVENT_INIT:
	return __local_event_init();
    case LOCAL_EVENT_EXEC:
	return __local_event_exec();
    }
}

void ICACHE_FLASH_ATTR
user_init() {

    /* either static or declared outside!! */
    static os_event_t __local_task_queue[8];
    
    /* mandatory initialization
       - microsecond timer as first thing to do */
#ifdef USE_US_TIMER
    system_timer_reinit();
#endif

    /* baud-rate */
    uart_div_modify(0 /* uart0 */, UART_CLK_FREQ / 115200);


    /* initialize taskqueue */
    system_os_task(__local_event_callback, VE_TASKPRIO_LOW,
		   __local_task_queue, __asize(__local_task_queue));


    /* post callback */
    system_os_post(VE_TASKPRIO_LOW, LOCAL_EVENT_INIT, 0);
}


/*
  Local variables: ***
  mode: c ***
  c-basic-offset: 4 ***
  End: ***
*/
